import '../../match_context.dart';
import '../../qr.dart';
import '../../types.dart';
import 'tree_builder.dart';
import 'tree_matcher.dart';
import 'tree_types.dart';

class RoutesTree {
  Tree _tree;
  final _matcher = TreeMatcher();

  void buildTree(List<QRouteBase> routes) {
    QR.history.clear();
    QR.currentRoute.fullPath = '';
    _tree = TreeBuilder().buildTree(routes);
    _matcher.tree = _tree;
    logTree();
    QR.log('Tree Built', isDebug: true);
  }

  void logTree() {
    QR.log(
        '${''.padRight(15, '-')} Project Route Tree ${''.padRight(15, '-')}');
    for (var route in _tree.routes) {
      route.printTree(0, _tree.routes.last == route, []);
    }
    QR.log(''.padRight(50, '-'));
  }

  MatchContext getMatch(String path) {
    return _matcher.getMatch(path);
  }

  String findPathFromName(String name, Map<String, dynamic> params) {
    var path = _tree.treeIndex[name];
    assert(path != null, 'Path name not found');
    final pathParams = <String, dynamic>{};

    // Search for component params
    for (var param in params.entries) {
      if (path.contains(':${param.key}')) {
        path = path.replaceAll(':${param.key}', param.value.toString());
      } else {
        pathParams.addEntries([param]);
      }
    }

    // Replace old component
    for (var param in QR.params.asMap.entries) {
      if (path.contains(':${param.key}')) {
        path = path.replaceAll(':${param.key}', param.value.value);
      }
    }

    if (pathParams.isNotEmpty) {
      path = '$path?';
      // Build the params
      for (var i = 0; i < pathParams.length; i++) {
        final param = pathParams.entries.toList()[i];
        path = '$path${param.key}=${param.value}';
        if (i != pathParams.length - 1) {
          path = '$path&';
        }
      }
    }
    return path;
  }
}
